#!/usr/bin/env python3


import os
import sys
import pathlib
import platform
import re
import uuid
import socket
import pickle
import webbrowser
import datetime
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.messagebox import askokcancel
from app.shell import WsShell
from tooltip import CreateToolTip
import settings
try:
    import winreg as wrg
except ModuleNotFoundError:
    pass


VERSION = '2.0'
HEADER = 1024




class CompanyDataForm():
    def __init__(self, window, controller):
        self.window = window
        self.controller = controller
        self._draw()

    def _draw(self):
        self.window.title("Dades de l'empresa")
        f = tk.Frame(self.window)

        f1 = tk.Frame(f)
        f11 = tk.Frame(f1)
        l1 = ttk.Label(f11, text='Nom / Raó Social')
        l1.pack()
        self.t1 = ttk.Entry(f11)
        self.t1.pack(padx=2, pady=2, fill=tk.X, expand=1)
        f11.pack(side=tk.LEFT, expand=1, fill=tk.X)
        f12 = tk.Frame(f1)
        l2 = ttk.Label(f12, text='NRT')
        l2.pack()
        self.t2 = ttk.Entry(f12, widt=12)
        self.t2.pack(padx=2, pady=2)
        f12.pack(side=tk.LEFT, fill=tk.X)
        f1.pack(expand=1, fill=tk.X, padx=5, pady=5)

        f2 = tk.Frame(f)
        f21 = tk.Frame(f2)
        l3 = ttk.Label(f21, text='Adreça')
        l3.pack()
        self.t3 = ttk.Entry(f21, width=60)
        self.t3.pack(padx=2, pady=2, fill=tk.X, expand=1)
        f21.pack(side=tk.LEFT, expand=1, fill=tk.X)
        f22 = tk.Frame(f2)
        l4 = ttk.Label(f22, text='Població')
        l4.pack()
        self.t4 = ttk.Entry(f22)
        self.t4.pack(padx=2, pady=2, fill=tk.X)
        f22.pack(side=tk.LEFT, fill=tk.X)
        f23 = tk.Frame(f2)
        l5 = ttk.Label(f23, text='CP')
        l5.pack()
        self.t5 = ttk.Entry(f23, width=10)
        self.t5.pack(padx=2, pady=2)
        f23.pack(side=tk.LEFT, fill=tk.X)
        f2.pack(expand=1, fill=tk.X, padx=5, pady=5)

        f3 = tk.Frame(f)
        f31 = tk.Frame(f3)
        l6 = ttk.Label(f31, text='Email')
        l6.pack()
        self.t6 = ttk.Entry(f31)
        self.t6.pack(padx=2, pady=2, fill=tk.X, expand=1)
        f31.pack(side=tk.LEFT, fill=tk.X, expand=1)
        f32 = tk.Frame(f3)
        l7 = ttk.Label(f32, text='Telèfon')
        l7.pack()
        self.t7 = ttk.Entry(f32)
        self.t7.pack(padx=2, pady=2, fill=tk.X)
        f32.pack(side=tk.LEFT, fill=tk.X)
        f3.pack(expand=1, fill=tk.X, padx=5, pady=5)

        f4 = tk.Frame(f)
        l8 = ttk.Label(f4, text='Persona de contacte')
        l8.pack()
        self.t8 = ttk.Entry(f4)
        self.t8.pack(padx=2, pady=2, fill=tk.X, expand=True)
        f4.pack(expand=True, fill=tk.X, padx=5, pady=5)

        f5 = tk.Frame(f)
        b1 = ttk.Button(f5, text='Acceptar', command=self._accept)
        b1.pack(side=tk.LEFT, padx=20)
        b2 = ttk.Button(f5, text='Cancelar', command=self._cancel)
        b2.pack(side=tk.RIGHT, padx=30, pady=20)
        f5.pack()

        f.pack(expand=1, fill=tk.X)
        self.set_modal()

    def set_modal(self):
        try:
            self.window.wait_visibility()
        except:
            pass
        try:
            self.window.focus_set()
        except:
            pass
        try:
            self.window.grab_set()
        except:
            pass
        try:
            self.window.wait_window(self.window)
        except:
            pass

    def _check_values(self):
        data = {}
        data['name'] = self.t1.get()
        data['nrt'] = self.t2.get()
        data['address'] = self.t3.get()
        data['city'] = self.t4.get()
        data['postalcode'] = self.t5.get()
        data['email'] = self.t6.get()
        data['phone'] = self.t7.get()
        data['pax'] = self.t8.get()

        if '' in data.values():
            return None
        else:
            return data

    def _accept(self):
        data = self._check_values()
        if data:
            self.controller.create_license(data)
            self.window.destroy()
        else:
            txt = "  PROBLEMA: S'han d'omplir tots els camps"
            txt = txt + " del formulari"
            print(txt)

    def _cancel(self):
        self.window.destroy()
        self.controller.close()


class MachineManager():
    
    def __init__(self, caller):
        self.caller = caller
        os = platform.system()
        if 'inux' in os:
            self.system_platform = 'Linux'
        elif 'indow' in os:
            self.system_platform = 'Windows'
        else:
            self.system_platform = None

    def get_key(self):
        if not self.system_platform:
            return None
        elif self.system_platform == 'Windows':
            return self._get_key_win()
        elif self.system_platform == 'Linux':
            return self._get_key_lin()

    def set_key(self, key):
        if not self.system_platform:
            return None
        elif self.system_platform == 'Windows':
            return self._set_key_win(key)
        elif self.system_platform == 'Linux':
            return self._set_key_lin(key)

    def _get_key_win(self):
        location = wrg.HKEY_CURRENT_USER
        try:
            soft = wrg.OpenKeyEx(location, r"SOFTWARE\\Mlmt\\")
            k = int(wrg.QueryValueEx(soft, "K")[0])
            if soft:
                wrg.CloseKey(soft)
            if k <= 0:
                self._erase_key_win()
                k = 0
        except:
            k = 0
        return k

    def erase_key(self):
        if not self.system_platform:
            return
        elif self.system_platform == 'Windows':
            self._erase_key_win()
        elif self.system_platform == 'Linux':
            self._erase_key_lin()

    def _erase_key_win(self):
        try:
            location = wrg.HKEY_CURRENT_USER
            soft = wrg.OpenKeyEx(location, r"SOFTWARE\\")
            key_1 = wrg.CreateKey(soft, "Mlmt")
            de_val = wrg.DeleteValue(key_1, "K")
            del_key = wrg.DeleteKey(key_1, "")
            if key_1:
                wrg.CloseKey(key_1)
            if soft:
                wrg.CloseKey(soft)
        except:
            pass

    def _get_key_lin(self):
        dirpath = pathlib.Path.home()
        filepath = os.path.join(dirpath, '.mlmt.conf')
        if os.path.isfile(filepath):
            f = open(filepath, 'r')
            k = int(f.read())
            f.close()
            if k <= 0:
                self._erase_key_lin()
                k = 0
        else:
            k = 0
        return k

    def _erase_key_lin(self):
        dirpath = pathlib.Path.home()
        filepath = os.path.join(dirpath, '.mlmt.conf')
        if os.path.isfile(filepath):
            os.remove(filepath)

    def _set_key_win(self, k):
        if not self._get_key_win() is str:
            location = wrg.HKEY_CURRENT_USER
            soft = wrg.OpenKeyEx(location, r"SOFTWARE\\")
            key_1 = wrg.CreateKey(soft, "Mlmt")
            wrg.SetValueEx(key_1, "K", 0, wrg.REG_SZ, str(k))
            if key_1:
                wrg.CloseKey(key_1)
            return True
        else:
            return False

    def _set_key_lin(self, k):
        dirpath = pathlib.Path.home()
        filepath = os.path.join(dirpath, '.mlmt.conf')
        if not self._get_key_lin():
            f = open(filepath, 'w')
            f.write(str(k))
            f.close()
            return True
        else:
            return False

    def get_mac_address(self):
        return ':'.join(re.findall('..', '%012x' % uuid.getnode()))

    def get_host_name(self):
        return socket.gethostname()

    def get_settings(self):
        s = {}
        s['key'] = self.get_key()
        s['mac'] = self.get_mac_address()
        s['host_name'] = self.get_host_name()
        s['os'] = self.system_platform
        s['pyversion'] = platform.python_version()
        return s


class AppManager():

    def __init__(self, caller):
        self.caller = caller
        p1 = os.path.join(self.caller.path, 'app')
        filepath = os.path.join(p1, 'sttg.xlsx')
        if os.path.isfile(filepath):
            self.filepath = filepath
        else:
            self.filepath = None

    def _get_workbook(self):
        import openpyxl
        return openpyxl.load_workbook(self.filepath)

    def set_session(self, value):
        wb = self._get_workbook()
        ws = wb['DATA']
        cell = ws.cell(5, 2)
        cell.value = value
        wb.save(self.filepath)

    def set_license(self, value):
        wb = self._get_workbook()
        ws = wb['DATA']
        cell = ws.cell(3, 2)
        cell.value = value
        wb.save(self.filepath)
        
    def set_key(self, value):
        wb = self._get_workbook()
        ws = wb['DATA']
        cell = ws.cell(4, 2)
        cell.value = value
        wb.save(self.filepath)

    def get_settings(self):
        if not self.filepath:
            return None
        else:
            wb = self._get_workbook()
            try:
                ws = wb['DATA']
            except:
                return None
            else:
                s = {}
                row = 0
                more = True
                while more:
                    row += 1
                    cell1 = ws.cell(row, 1)
                    cell2 = ws.cell(row, 2)
                    if cell1.value:
                        s[cell1.value] = cell2.value
                    else:
                        more = False
                return s


class BaseController(object):

    def __init__(self, caller):
        self.caller = caller
        self.path = self.caller.path
        self._manage_dirs()
        if self._check_app_files() is False:
            self.caller.damaged = True
            return
            
        self.machine_manager = MachineManager(caller=self)
        self.app_manager = AppManager(caller=self)

    def _manage_dirs(self):
        for item in settings.APP_FOLDERS:
            path = os.path.join(self.path, item[0])
            if not os.path.exists(path):
                os.mkdir(path)
            else:
                if item[1] is True:      # empty folder
                    try:
                        files = os.listdir(path)
                        for file in files:
                            file_path = os.path.join(path, file)
                            if os.path.isfile(file_path):
                                os.remove(file_path)
                    except:
                        pass
                    
    def _check_app_files(self):
        result = True
        path = os.path.join(self.path, 'app')
        for item in settings.APP_FILES:
            mypath = os.path.join(path, item)
            if not os.path.isfile(mypath):
                result = False
                break
        return result
        
    def _build_filename_from_datetime(self, termination):
        value = datetime.datetime.now()
        y = str(value.year)
        m = str(value.month)
        d = str(value.day)
        mi = str(value.minute)
        s = str(value.second)
        return y + m + d + mi + s + termination
        

    def _update_settings(self):
        self.msttg = self.machine_manager.get_settings()
        self.asttg = self.app_manager.get_settings()

    def on_open(self):
        self.app_manager.set_session(0)
        self._update_settings()
        print("Comprovant configuració ...")
        response = self._check_config()
        if response[0] == 0:
            print(response[1])
            return 0
        else:
            print("Configuració OK")
            return response[0]

    def _get_log_vals(self):
        vals = {'app': self.asttg['app'],
                'app_version': self.asttg['version'],
                'machine_key': self.msttg['key'],
                'license': self.asttg['license'],
                'mac': self.msttg['mac'],
                'name': self.msttg['host_name'],
                'platform': self.msttg['os'],
                'pyversion': self.msttg['pyversion'],
                'session': self.asttg['session'],
                'follow': self.asttg['follow']}
        return vals

    def install(self):
        response = self._send_socket(command=1)
        if not response:
            print("Error 101 - No s'ha pogut connectar amb el socket server")
            self.caller.set_status(settings.STATUS_INITIAL)
        else:
            if response[0] == 0:
                self.caller.set_status(settings.STATUS_INITIAL)
            if response[0] == 3:
                print(response[1][0])
                self.machine_manager.set_key(response[1][1])
                self.caller.set_status(settings.STATUS_INITIAL)
            elif response[0] == 4:
                print(response[1])
                self.caller.set_status(STATUS_ONLY_EXIT)
            elif response[0] == 5:
                txt = "Error 113 - El codi de'estació de treball és"
                txt = txt + " incorrecte. Torni a provar d'entrar."
                print(txt)
                mm = self.machine_manager
                mm.erase_key()
                self.caller.set_status(settings.STATUS_INITIAL)
            elif response[0] == 1:
                if response[1][2] == 2:
                    self._get_activate_txt(False)
                    self.caller.set_status(settings.STATUS_INITIAL)
                elif response[1][2] == 0:
                    session = response[1][3]
                    license_ = response[1][1]
                    self.app_manager.set_session(session)
                    self.app_manager.set_license(license_)
                    self.app_manager.set_key(self.msttg['key'])
                    self._update_settings()
                    validity = response[1][4]
                    self._show_info(validity)
                    self.caller.set_status(settings.STATUS_ACTIVE)
            elif response[0] == 6:
                # Cal crear llicència
                if response[1][0]:   # S'ha creat machine key
                    mm = self.machine_manager
                    mm.set_key(response[1][0])
                w = tk.Toplevel()
                form = CompanyDataForm(w, self)

    def _on_fake_open(self, old_version=False):
        if old_version:
            body = "La versió utilitzada del programa no és la darrera."
            body = body + " Si desitja instal·lar la versió actual,"
        else:
            body = "La llicència del programa no és correcta."
            body = body + " Si desitja instal·lar una nova llicència,"

        body = body + " la pot trobar a  www.malamute.tech."
        body = body + " Vol accedir al lloc web?"

        head = self.asttg['appname']
        response = self.caller.ask_ok_cancel(head, body)
        if response:
            self.open_web()
            self.close()
        else:
            self.caller.set_status(settings.STATUS_ONLY_EXIT)
            
    def on_damaged_app(self):
        body = "El programa està danyat. Cal tornar a instalar-lo."
        body = body + " El pot trobar a  www.malamute.tech."
        body = body + " Vol accedir al lloc web?"

        head = 'Problema'
        response = self.caller.ask_ok_cancel(head, body)
        if response:
            self.open_web()
            self.close()
        else:
            self.caller.set_status(settings.STATUS_ONLY_EXIT)

    def create_license(self, data):
        self._update_settings()
        response = self._send_socket(command=6, data=data)
        if not response:
            print("Error 101 - No s'ha pogut connectar amb el socket server")
            self.caller.set_status(settings.STATUS_INITIAL)
        elif response[0] == 0:
            print(response[1])
            self.caller.set_status(settings.STATUS_INITIAL)
        elif response[0] == 1:
            self.app_manager.set_license(response[1][1])
            self.app_manager.set_key(self.msttg['key'])
            self._update_settings()
            if response[1][2]== 1:
                self._get_activate_txt(True)
                self.caller.set_status(settings.STATUS_ONLY_EXIT)
            else:
                print()
                print('Benvingut al període de prova, que finalitza el ' + response[1][3] )
                self.caller.set_status(settings.STATUS_ACTIVE)
                

    def login(self):
        response = self._send_socket(command=4)
        if not response:
            msg = "Error 000 - No es pot connectar amb el socket server."
            msg = msg + " Verifiqui la seva connexió a internet."
            print(msg)
        elif response[0] == 0:
            msg = response[1]
            print(msg)
        elif response[0] == 2:
            msg = response[1]
            print(msg)
            self._on_fake_open(old_version=False)
        elif response[0] == 4:
            msg = response[1]
            print(msg)
            self._on_fake_open(old_version=True)
        elif response[0] == 5:
            msg = "Error 001 - El codi de'estació de treball és incorrecte"
            print(msg)
            self.machine_manager.erase_key()
            self.caller.set_status(settings.STATUS_INITIAL)
        elif response[0] == 1:
            self._get_activate_txt(False)
            self.caller.set_status(settings.STATUS_ONLY_EXIT)
        elif response[0] == 3:
            session = response[1]
            self.app_manager.set_session(session)
            self.caller.set_status(settings.STATUS_ACTIVE)
            self._update_settings()
            validity = response[2]
            self._show_info(validity)
            
    def _show_info(self, validity=None):
        code = str(self.asttg['license']) + '.'
        code = code + str(self.msttg['key']) + '.'
        code = code + self.msttg['pyversion']
        print(110 * '-')
        txt = "La seva llicència està activa. El seu codi identificatiu és: "
        txt = txt + code
        if validity:
            txt2 = ".  I la seva data de caducitat és: " + validity
            txt = txt + txt2
        print(txt)
        print(110 * '-')
        
    def _show_leaflet(self):
        m = os.path.join(self.path, 'app')
        mypath = os.path.join(m, 'leaflet.pdf')
        self._show_file_on_browser(mypath)

    def _show_file_on_browser(self, filepath):
        try:
            webbrowser.open_new_tab(filepath)
        except:
            txt = "Error 801 - No s'ha pogut mostrar l'arxiu " + filepath
            print(txt)
            
    def _get_app_working_folder_path(self):
        if settings.APP_WORKING_FOLDER:
            path = os.path.join(self.caller.path, 
                                settings.APP_WORKING_FOLDER)
            if not os.path.isdir(path):
                path = None
        else:
            path = None
        return path

    def _show_xlsx_file(self, filepath):
        if self.msttg['os'] == 'Linux':
            if self._show_xlsx_file_linux(filepath) is False:
                self._show_file_on_browser(filepath)
        elif self.msttg['os'] == 'Windows':
            if self._show_xlsx_file_windows(filepath) is False:
                self._show_file_on_browser(filepath)
        else:
            pass
            
        
    def _show_xlsx_file_windows(self, filepath):
        try:
            # command = 'start EXCEL.EXE "' + filepath + '"'
            # os.system(command)
            os.startfile(filepath)
            return True
        except:
            try:
                command = 'soffice -o "' + filepath + '"'
                os.system(command)
                return True
            except:
                return False
                
        
    def _show_xlsx_file_linux(self, filepath):
        try:
            command = 'soffice -o "' + filepath + '"'
            os.system(command)
            return True
        except:
            return False
    
    
        

    def _check_config(self):
        if self.asttg is None:
            txt = "PROBLEMA 001: No es pot accedir a l'arxiu de "
            txt = txt + " configuració"
            return (0, txt)

        if self.asttg == {}:
            txt = "PROBLEMA 002: L'arxiu de configuració no té dades"
            return (0, txt)

        keys = ('key', 'license', 'session', 'app', 'version', 'follow',
                'appname', 'web')

        ok = True
        for k in keys:
            if k not in self.asttg:
                ok = False
                break
        if not ok:
            txt = "PROBLEMA 003: L'arxiu de configuració té dades imcompletes"
            return (0, txt)

        if self.msttg['key'] == 0:
            if self.asttg['license'] == 0:
                if self.asttg['key'] == 0:
                    return (1, 'install')
                else:
                    txt = "PROBLEMA 004: No es pot arrancar el programa."
                    txt = txt + " Decarregui'l de nou a la nostra web:\n"
                    txt = txt + " www.malamute.tech"
                    return (0, txt)
            else:
                txt = "PROBLEMA 005: No es pot arrancar el programa."
                txt = txt + " Decarregui'l de nou a la nostra web:\n"
                txt = txt + " www.malamute.tech"
                return (0, txt)
        else:
            if self.asttg['license'] == 0:
                if self.asttg['key'] == 0:
                    return (1, 'install')
                else:
                    txt = "PROBLEMA 006: No es pot arrancar el programa."
                    txt = txt + " Decarregui'l de nou a la nostra web:\n"
                    txt = txt + " www.malamute.tech"
                    return (0, txt)
            else:
                if self.asttg['key'] == 0:
                    return (2, 'login')
                else:
                    if self.asttg['key'] == self.msttg['key']:
                        return (2, 'login')
                    else:
                        txt = "PROBLEMA 007: No es pot arrancar el"
                        txt = txt + " programa. Decarregui'l de nou a"
                        txt = txt + " la nostra web:\n"
                        txt = txt + " www.malamute.tech"
                        return (0, txt)

    def _send_socket(self, command, data=0):
        d = {'log_vals': self._get_log_vals(),
             'command': command,
             'args': data}
        # ............................................................
        if settings.DEBUG:
            print('ENVIAT:')
            print()
            print(d)
            print()
        # .............................................................

        msg = pickle.dumps(d)
        host = settings.HOST
        port = settings.PORT
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        received = None

        try:
            s.connect((host, port))
            s.sendall(msg)
            s.shutdown(socket.SHUT_WR)
            chunks = []
            while True:
                data = s.recv(HEADER)
                if not data:
                    break
                chunks.append(data)
            received = b''.join(chunks)
            received = pickle.loads(received)
        finally:
            s.close()
            # .............................................................
            if settings.DEBUG:
                print('REBUT:')
                print()
                print(received)
                print()
            # .............................................................
            return received

    def open_web(self):
        webbrowser.open_new_tab(settings.WEB)

    def _get_activate_txt(self, first=False):
        self._update_settings()
        if first:
            data_ = {'first': True}
        else:
            data_ = {'first': False}
        response = self._send_socket(command=3, data=data_)
        error = True
        if response:
            if type(response) is str:
                error = False
                print(response)
        if error:
            sett = self.app_manager.get_settings()
            li = str(sett['license'])
            msg = "Error 202 - No s'ha pogut rebre el missatge per"
            msg = msg + " l'activació de la llicència L-{}."
            msg = msg + " Contacti amb info@malamute.tech"
            msg.format(li)
            print()
            print(msg)

    def close(self):
        self.caller._exit()
        sys.exit()


class BaseView(object):
    
    def __init__(self, path):
        self.path = path
        self.damaged = False
        
    def _get_icons(self):
        iconspath = os.path.join(self.path, 'icons')
        icons = {}
        for item in settings.ICONS:
            filepath = os.path.join(iconspath, item + '.gif')
            icons[item] = tk.PhotoImage(file=filepath)
        return icons
        
    def _draw(self):
        self._draw_header()
        self._draw_app_buttons()
        self._draw_shell()
        self._draw_footer()
        self.set_status(settings.STATUS_INITIAL)
        self.top.mainloop()
        
    def _draw_header(self):
        self.top = tk.Tk()
        self.top.resizable(False, False)
        self.top.title(settings.APP_NAME + ' ' + settings.APP_VERSION)
        self.icons = self._get_icons()
        style = ttk.Style(self.top)
        style.theme_use(settings.DEFAULT_STYLE)
        
    def _draw_app_buttons(self):
        pass

    def _draw_footer(self):
        f1 = tk.Frame(self.top)
        self.btn_enter = ttk.Button(f1, text="Entrar", 
                                    command=self._start,
                                    image=self.icons['apply'])
        CreateToolTip(self.btn_enter, "Entrar")
        self.btn_enter.pack(side=tk.LEFT, pady=0, padx=2.5)
        self.btn_exit = ttk.Button(f1, text='Sortir',
                                   command=self._exit,
                                   image=self.icons['exit'])
        CreateToolTip(self.btn_exit, "Tancar l'aplicatiu")
        self.btn_exit.pack(side=tk.LEFT, pady=0, padx=2.5)
        self.btn_print = ttk.Button(f1, text='Imprimir',
                                    command=self._print,
                                    image=self.icons['printer'])
        self.btn_print.pack(side=tk.LEFT, pady=0, padx=2.5)
        CreateToolTip(self.btn_print, "Imprimir consola")
        self.btn_leaflet = ttk.Button(f1, text='Documentacio',
                                    command=self.controller._show_leaflet,
                                    image=self.icons['leaflet'])
        self.btn_leaflet.pack(side=tk.LEFT, pady=0, padx=2.5)
        CreateToolTip(self.btn_leaflet, "Mostrar documentació")
        self.btn_web = ttk.Button(f1, text='Web',
                                  command=self._show_web,
                                  image=self.icons['web'])
        CreateToolTip(self.btn_web, "Accedir a www.malamute.tech")
        self.btn_web.pack(side=tk.LEFT, pady=0, padx=2.5)
        mark = tk.Label(f1, text='www.malamute.tech',
                        fg='navy',
                        font=("Arial", 10))
        mark.pack(side=tk.RIGHT, padx=5)
        appname = tk.Label(f1, text=settings.APP_NAME,
                           fg='red',
                           font=("Arial", 13))
        appname.pack(side=tk.RIGHT, padx=8)
        f1.pack(padx=5, pady=5, fill=tk.BOTH)
        
    def _show_web(self):
        self.controller.open_web()
        

    def _draw_shell(self):
        self.text = WsShell(self.top, height=15)
        self.text.pack(padx=5, pady=5)

    def set_status(self, value):
        '''
        A desenvolupar amb les subclasses
        '''

    def _exit(self):
        sys.exit()
        

    def _print(self):
        txt = self.text.get_text()
        path = os.path.join(self.path, 'tmp')
        if not os.path.exists(path):
            os.mkdir(path)
        filename = self.controller._build_filename_from_datetime('.txt')
        path = os.path.join(path, 'missatge_' + filename)
        f = open(path, 'w')
        f.write(txt)
        f.close()
        webbrowser.open_new_tab(path)

    def _start(self):
        self.text.clear()
        if self.damaged:
            self.controller.on_damaged_app()
        else:
            status = self.controller.on_open()
            if status == 0:
                head = 'Descarregar'
                body = "Desitja accedir a la nostra web per obtenir un"
                body = body + " paquet del programa?"
                response = self.ask_ok_cancel(head, body)
                if response:
                    self.controller.open_web()
                    self._exit()
                else:
                    self._exit()
            elif status == 1:  # install
                print("Creació de llicència ...")
                txt = "El procés de creació de llicència recuperarà"
                txt = txt + " l'antiga si ja en va crear una anteriorment."
                txt = txt + " Si no és el cas, se li demanaran les dades"
                txt = txt + " de l'empresa per poder crear-ne una de nova."
                print()
                print(txt)
                print()
                head = 'Llicència'
                body = "Desitja iniciar el procés de creació de llicència?"
                response = self.ask_ok_cancel(head, body)
                if response:
                    self.controller.install()
                else:
                    self._exit()
            elif status == 2:   # login
                self.controller.login()

    def ask_ok_cancel(self, head, body):
        return askokcancel(head, body)

    def _select_open_file(self, file=None, initialdir="/",
                          title='Seleccioni arxiu',
                          filetypes=[('arxius XML', '*.xml')]):
        from tkinter.filedialog import askopenfilename

        return askopenfilename(initialdir=initialdir,
                               title=title, filetypes=filetypes)
                               
    def _set_status(self, value):
        if value == settings.STATUS_INITIAL:
            self.btn_enter.state(["!disabled"])
            self.btn_exit.state(["!disabled"])
            self.btn_print.state(["disabled"])
            self.btn_leaflet.state(["disabled"])
            self.btn_web.state(["disabled"])
        elif value == settings.STATUS_ONLY_EXIT:
            self.btn_enter.state(["disabled"])
            self.btn_exit.state(["!disabled"])
            self.btn_print.state(["!disabled"])
            self.btn_leaflet.state(["!disabled"])
            self.btn_web.state(["!disabled"])
        elif value == settings.STATUS_ACTIVE:
            self.btn_enter.state(["disabled"])
            self.btn_exit.state(["!disabled"])
            self.btn_print.state(["!disabled"])
            self.btn_leaflet.state(["!disabled"])
            self.btn_web.state(["!disabled"])
