"""A ScrolledText widget feels like a text widget but also has a
vertical scroll bar on its right.  (Later, options may be added to
add a horizontal bar as well, to make the bars disappear
automatically when not needed, to move them to the other side of the
window, etc.)
Configuration options are passed to the Text widget.
A Frame widget is inserted between the master and the text, to hold
the Scrollbar widget.
Most methods calls are inherited from the Text widget; Pack, Grid and
Place methods are redirected to the Frame widget however.
"""

# __all__ = ['WsttkScrolledText']

import sys
import time
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.constants import RIGHT, LEFT, Y, BOTH


class RedirectText(object):

    def __init__(self, text_ctrl):
        """Constructor"""
        self.output = text_ctrl

    def write(self, string):

        self.output.insert(tk.END, string)
        self.output.update_idletasks()
        time.sleep(0.1)

    def flush(self):
        pass

class WsShell(tk.Text):

    def __init__(self, master=None, **kw):
        self.frame = ttk.Frame(master)
        self.vbar = ttk.Scrollbar(self.frame)
        self.vbar.pack(side=RIGHT, fill=Y)

        kw.update({'yscrollcommand': self.vbar.set})
        tk.Text.__init__(self, self.frame, bg='black', fg='green2',
                                      font=(None, 9),**kw)

                # redirect stdout
        redir = RedirectText(self)
        sys.stdout = redir

        self.pack(side=LEFT, fill=BOTH, expand=True)
        self.vbar['command'] = self.yview

        # Copy geometry methods of self.frame without overriding Text
        # methods -- hack!
        text_meths = vars(tk.Text).keys()
        methods = vars(tk.Pack).keys() | vars(tk.Grid).keys() | vars(tk.Place).keys()
        methods = methods.difference(text_meths)

        for m in methods:
            if m[0] != '_' and m != 'config' and m != 'configure':
                setattr(self, m, getattr(self.frame, m))

        self.bind('<<Modified>>',self._show_end)

    def _show_end(self, event=None):
        self.see(tk.END)
        self.edit_modified(0)

    def get_text(self):
        return self.get(1.0, tk.END)
        
    def clear(self):
        self.delete(1.0, tk.END)
        

    def __str__(self):
        return str(self.frame)

    def callback():
        text.see(END)
        text.edit_modified(0)

'''
def example():
    from tkinter.constants import END

    stext = ScrolledText(bg='white', height=10)
    stext.insert(END, __doc__)
    stext.pack(fill=BOTH, side=LEFT, expand=True)
    stext.focus_set()
    stext.mainloop()

if __name__ == "__main__":
    example()
'''
