#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import os
import shutil
import xml.etree.ElementTree as ET
from pathlib import Path
import tkinter as tk
import tkinter.ttk as ttk
from basecontroller import BaseController, BaseView
import openpyxl
import settings


class XmlFileManager(object):

    def __init__(self, caller, filepath):
        self.caller = caller
        self.xml_filepath = filepath
        f_ = Path(self.xml_filepath).name
        f_ = f_.split('.')[0]
        self.xlsx_filename = f_ + '.xlsx'

    def import_to_xlsx(self):
        self._create_xlsx()
        self._write_filepath()
        data = self.get_data()
        self._write_company_data(data[0])
        self._write_employees_data(data[1])
        self.workbook.save(self.to_)
        txt = "S'ha generat el nou arxiu " + self.to_
        print()
        print(txt)
        self.caller._show_xlsx_file(self.to_)

    def _create_xlsx(self):
        path = os.path.join(self.caller.path, 'app')
        from_ = os.path.join(path, 'template.xlsx')
        path = os.path.join(self.caller.path, 'DECLARACIO_ACTUAL')
        self.to_ = os.path.join(path, self.xlsx_filename)
        shutil.copyfile(from_, self.to_)
        self.workbook = openpyxl.load_workbook(self.to_)

    def _write_filepath(self):
        worksheet = self.workbook["DEC"]
        worksheet['B1'] = self.xml_filepath

    def get_data(self):
        tree = ET.parse(self.xml_filepath)
        root = tree.getroot()
        company_data = self._get_company_data(root)
        employees = self._get_employees_data(root)
        return [company_data, employees]

    def _get_company_data(self, root):
        company_data = {'documentNumber': None,
                        'month': None,
                        'year': None,
                        'companyId': None,
                        'nameAndSurname': None}
        for child in root:
            if "recordDetails" in child.tag:
                for schild in child:
                    for key in company_data:
                        if key in schild.tag:
                            company_data[key] = schild.text
        return company_data

    def _get_employees_data(self, root):
        employees = {}
        employee_data = {'id': None,
                         'nameAndSurname': None,
                         'orderNumber': None,
                         'joinDate': None}
        for child in root:
            if "recordDetails" in child.tag:
                for schild in child:
                    if "quotationInsuredList" in schild.tag:
                        data = {}
                        for sschild in schild:
                            for key in employee_data:
                                if key in sschild.tag:
                                    data[key] = sschild.text
                        if data['orderNumber']:
                            employees[int(data['orderNumber'])] = data
        return employees

    def _write_company_data(self, data):
        rows = (('nameAndSurname', 3), ('companyId', 4),
                ('documentNumber', 5),
                ('year', 6), ('month', 7))

        worksheet = self.workbook["DEC"]
        for r in rows:
            cell = worksheet.cell(row=r[1], column=2)
            cell.value = data[r[0]]

    def _write_employees_data(self, data):
        worksheet = self.workbook["DEC"]
        keys = sorted(data.keys())
        row = 11

        cols = (('nameAndSurname', 1), ('orderNumber', 2), ('id', 3),
                ('joinDate', 4))

        for k in keys:
            row += 1
            for col in cols:
                cell = worksheet.cell(row, col[1])
                cell.value = data[k][col[0]]

            cell = worksheet.cell(row, 14)
            f = "=E{r} + G{r} + I{r} + J{r} + K{r} + L{r} + M{r}"
            formula = f.format(r=str(row))
            cell.value = formula
            # cell = worksheet.cell(10, 14)
            # formula = "=SUM(N12:N" + str(row)
            # cell.value = formula


class ThisController(BaseController):

    def __init__(self, caller):
        super().__init__(caller)

    def _get_xmlns_prefix(self, filepath, text):
        response = None
        for key, value in ET.iterparse(filepath,
                                       events=['start', 'start-ns']):
            if key == 'start-ns':
                if text in value[1]:
                    response = value[0]
        return response

    def import_xml_file(self, dirpath):
        f = XmlFileManager(self, dirpath)
        f.import_to_xlsx()

    def create_xml_file(self, filepath):
        self.xlsx_manager = XlsxFileManager(self, filepath)
        self.basepath = os.path.abspath(os.path.dirname(filepath))
        path = self.xlsx_manager.get_xml_filepath()
        self.xml_manager = XmlFileManager(self, path)
        if not self._check_data():
            return

        d = self.xlsx_manager.upload_employees_data()
        errors = self._check_employees_data(d)
        if errors is False:
            args = []
            response = self._send_socket(command=5, data=args)
            self._manage_xml_response(response)

    def _check_data(self):
        path = self.xlsx_manager.get_xml_filepath()
        if not os.path.isfile(path):
            txt = "Error 001 - L'adreça de l'arxiu XML original"
            txt = txt + " (cel·la B1) no és correcta"
            print(txt)
            return False

        data = self.xml_manager.get_data()
        if not self._check_doc_code(data[0]):
            txt = "Error 002 - El codi de document (cel·la B5)"
            txt = txt + " no és correcte"
            print(txt)
            return False

        if not self._check_employees(data[1]):
            txt = "Error 003 - La llista de empleats no és correcta"
            print(txt)
            return False

        return True

    def _check_doc_code(self, data):
        code_xml = data['documentNumber']
        code_xlsx = self.xlsx_manager.get_doc_code()
        if code_xml != code_xlsx:
            return False
        else:
            return True

    def _check_employees(self, data):
        response = True
        xml_e = []
        for key in data:
            xml_e.append((key, data[key]['id']))

        xlsx_e = self.xlsx_manager.get_employees()
        if set(xml_e) - set(xlsx_e):
            response = False
        if set(xlsx_e) - set(xml_e):
            response = False
        return response

    def _check_employees_data(self, data):
        response = self._send_socket(command=2, data=data)
        if not response:
            txt = "Error 101 - No s'ha pogut connectar amb el servidor"
            print(txt)
            return True
        else:
            if response[0] != 33:
                print(response[1])
                return True
            else:
                if len(response[1]) > 0:
                    print("Error 033:")
                    for item in response[1]:
                        print('   - ' + item)
                    return True
                else:
                    return False

    def _manage_xml_response(self, response):
        if not response:
            txt = "Error 101 - No s'ha pogut connectar amb el servidor"
            print(txt)
        else:
            if response[0] in (0, 1, 2):
                txt = response[1]
                print(txt)
            else:
                self._bypass()

    def _bypass(self):
        da = self.xlsx_manager.upload_employees_data()
        filepath = self.xlsx_manager.get_xml_filepath()
        self.xlsx_manager.close()
        self._modify_xml(filepath, da)

    def _modify_xml(self, filepath, data):
        text = "companyquotation"
        ns = self._get_xmlns_prefix(filepath, text)
        if ns:
            ns = ns + ":"
        else:
            ns = ""

        f = open(filepath, 'r')
        txt = f.read()
        f.close()
        for order in data:
            pos0 = txt.find("orderNumber>" + str(order))
            pos1 = txt.find("/" + ns + "cno>", pos0)
            # La CASS ho pot enviar en 2 formats
            if pos1 == -1:   # No ho troba
                pos1 = txt.find("cno/>", pos0)
                pos3 = pos1 + 4 + len(ns)
            else:
                pos3 = pos1 + 5 + len(ns)
            new = ""
            for key in data[order]:
                if key not in ('SBAH', 'ARVH'):
                    new = new + "<" + ns + "quotationConceptList>"
                    new = new + "<" + ns + "code>" + key + "</" + ns + "code>"
                    new = new + "<" + ns + "salary>" + str(data[order][key]) + "</" + ns + "salary>"
                    if key in ('SBA', 'ARV'):
                        keyh = key + "H"
                        new = new + "<" + ns + "hours>" + str(data[order][keyh]) + "</" + ns + "hours>"
                    new = new + "</" + ns + "quotationConceptList>"
            txt = txt[:pos3] + new + txt[pos3:]

        filename = Path(filepath).name
        f = filename.split(".")[0] + "_OUTPUT.xml"
        new_filepath = os.path.join(self.basepath, f)
        f2 = open(new_filepath, 'w')
        f2.write(txt)
        f2.close()
        txt = "S'ha generat l'arxiu " + new_filepath + ". "
        txt = txt + "A continuació es presenta el resum del fitxer."
        print(txt)
        self._create_xml_summary(new_filepath)

    def _create_xml_summary(self, filepath):
        tree = ET.parse(filepath)
        root = tree.getroot()
        data = {'workers': {}}
        for child in root:
            if "recordDetails" in child.tag:
                for schild in child:
                    if "documentNumber" in schild.tag:
                        data['declaration'] = schild.text
                    elif "month" in schild.tag:
                        data['month'] = schild.text
                    elif "year" in schild.tag:
                        data['year'] = schild.text
                    elif "companyId" in schild.tag:
                        data['code'] = schild.text
                    elif "nameAndSurname" in schild.tag:
                        data['company'] = schild.text
                    elif "quotationInsuredList" in schild.tag:
                        e = {'order': None, 'code': None, 'name': None,
                             'join': None, 'items': {}}
                        for sschild in schild:
                            if "orderNumber" in sschild.tag:
                                e['order'] = sschild.text
                                current = e['order']
                            elif "id" in sschild.tag:
                                e['code'] = sschild.text
                            elif "joinDate" in sschild.tag:
                                e['join'] = sschild.text
                            elif "nameAndSurname" in sschild.tag:
                                e['name'] = sschild.text
                            elif "quotationConceptList" in sschild.tag:
                                if current not in data['workers']:
                                    data['workers'][current] = e
                                for ssschild in sschild:
                                    if "code" in ssschild.tag:
                                        currcode = ssschild.text
                                        data['workers'][current]['items'][currcode] = {}
                                    elif "salary" in ssschild.tag:
                                        data['workers'][current]['items'][currcode]['salary'] = ssschild.text
                                    elif "hours" in ssschild.tag:
                                        data['workers'][current]['items'][currcode]['hours'] = ssschild.text

        filepath = os.path.join(self.basepath, 'resum_declaracio.txt')
        f = open(filepath, 'w')
        t = "DECLARACIÓ: {}   PERÍODE: {}/{}\n".format(data['declaration'],
                                                       data['month'],
                                                       data['year'])
        f.write(t)
        t = "EMPRESA: {}\nNÚM. CASS: {}\n".format(data['company'],
                                                  data['code'])
        f.write(t)
        f.write('\n')
        for order in data['workers']:
            f.write(50 * '.' + '\n')
            t = "  {}  {}  {}  {}\n".format(order,
                                            data['workers'][order]['name'],
                                            data['workers'][order]['code'],
                                            data['workers'][order]['join'])
            f.write(t)
            for item in data['workers'][order]['items']:
                if 'hours' in data['workers'][order]['items'][item]:
                    t = "         {}:  {} euros  ({} hores)\n".format(item,
                                                                      data['workers'][order]['items'][item]['salary'],
                                                                      data['workers'][order]['items'][item]['hours'])
                else:
                    t = "         {}:  {} euros\n".format(item,
                                                          data['workers'][order]['items'][item]['salary'])
                f.write(t)
        f.close()
        self._show_file_on_browser(filepath)


class XlsxFileManager():

    def __init__(self, caller, filepath):
        self.caller = caller
        self.filepath = filepath
        self.workbook = openpyxl.load_workbook(self.filepath)

    def get_xml_filepath(self):
        worksheet = self.workbook["DEC"]
        return worksheet['B1'].value

    def get_doc_code(self):
        worksheet = self.workbook["DEC"]
        return worksheet['B5'].value

    def get_employees(self):
        worksheet = self.workbook["DEC"]
        xlsx_e = []
        row = 11
        end = False
        while not end:
            row += 1
            order = worksheet['B' + str(row)].value
            if not order:
                end = True
            else:
                id_ = worksheet['C' + str(row)].value
                xlsx_e.append((int(order), id_))
        return xlsx_e

    def upload_employees_data(self):
        data = {}
        cols = (('SBA', 'E'), ('SBAH', 'F'), ('ARV', 'G'),
                ('ARVH', 'H'), ('RES', 'I'),
                ('CPP', 'J'), ('PRE', 'K'), ('CEC', 'L'), ('CEI', 'M'))
        worksheet = self.workbook["DEC"]
        row = 11
        end = False
        while not end:
            row += 1
            order = worksheet['B' + str(row)].value
            if not order:
                end = True
            else:
                data[order] = {'SBA': 0, 'SBAH': 0}
                for col in cols:
                    value = worksheet[col[1] + str(row)].value
                    if value:
                        data[order][col[0]] = value
                        if col[0] == 'ARV':
                            data[order]['ARVH'] = 0
        return data
        
    def close(self):
        self.workbook.close()


class MainView(BaseView):

    folders = (('tmp', True),
               ('DECLARACIO_ACTUAL', False))

    app_files = ('template.xlsx', 'sttg.xlsx')

    def __init__(self, path):
        super().__init__(path)
        self.controller = ThisController(caller=self)
        self._draw()

    def _draw_app_buttons(self):

        f2 = tk.Frame(self.top)
        self.b4 = ttk.Button(f2, text="Importar nou fitxer CASS",
                             command=self._import)
        self.b4.state(["disabled"])
        self.b4.pack(side=tk.LEFT, pady=0, padx=5)
        self.b5 = ttk.Button(f2, text="Generar nou fitxer CASS",
                             command=self._export)
        self.b5.pack(side=tk.LEFT, pady=0, padx=5)
        self.b5.state(["disabled"])
        f2.pack(padx=5, pady=5)

    def _import(self):
        self.text.clear()
        initialdir_ = self.controller._get_app_working_folder_path()
        if not initialdir_:
            initialdir_ = '/'
        title_ ="Seleccioni arxiu"
        filepath = self._select_open_file(initialdir=initialdir_,
                                          title=title_,
                                          filetypes=[('arxius XML',
                                                      '*.xml')])
        if filepath:
            self.controller.import_xml_file(filepath)

    def set_status(self, value):
        self._set_status(value)
        if value == settings.STATUS_INITIAL:
            self.b4.state(["disabled"])
            self.b5.state(["disabled"])
        elif value == settings.STATUS_ONLY_EXIT:
            self.b4.state(["disabled"])
            self.b5.state(["disabled"])
        elif value == settings.STATUS_ACTIVE:
            self.b4.state(["!disabled"])
            self.b5.state(["!disabled"])

    def _export(self):
        self.text.clear()
        initialdir_ = self.controller._get_app_working_folder_path()
        if not initialdir_:
            initialdir_ = '/'
        title_ = "Seleccioni l'arxiu XLSX a processar"
        filepath = self._select_open_file(initialdir=initialdir_,
                                          title=title_,
                                          filetypes=[('arxius XLSX',
                                                      '*.xlsx')])
        if filepath:
            print("Processant l'arxiu " + filepath + " ...\n")
            self.controller.create_xml_file(filepath)
